#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
# Author:  Werner F. Bruhin
# Purpose: Application constants
# Created: 06/04/2012

import wx

# language domain
langDomain = "videomass"
# languages you want to support
supLang = {"en": wx.LANGUAGE_ENGLISH,
           "it": wx.LANGUAGE_ITALIAN,
          }
